local TileVO = require "main.tile.TileVO"

local BoardModel = {}

BoardModel.LEVEL_DATA_FOLDER = "C:/Nanogram/";
BoardModel.BLOCK_SIZE = 20;
BoardModel.BOARD_TILES_HOR = 35;
BoardModel.BOARD_TILES_VERT = 35;
BoardModel.screenW 	= tonumber(sys.get_config("display.width"));
BoardModel.screenH 	= tonumber(sys.get_config("display.height"));
BoardModel.BOARD_WIDTH = BoardModel.BLOCK_SIZE * BoardModel.BOARD_TILES_HOR;
BoardModel.BOARD_HEIGHT = BoardModel.BLOCK_SIZE * BoardModel.BOARD_TILES_VERT;

local side_edge = 150 + (BoardModel.screenW - BoardModel.BOARD_WIDTH) / 2 + (BoardModel.BLOCK_SIZE / 2);
local bottom_edge = (BoardModel.screenH - BoardModel.BOARD_HEIGHT) / 2 + (BoardModel.BLOCK_SIZE / 2);

BoardModel.GemsTypes = { "TileFactory" }

function BoardModel.get_tile_world_position(x, y)
	local pos = vmath.vector3(0, 0, 0);
	pos.x = side_edge + BoardModel.BLOCK_SIZE * x;
	pos.y = bottom_edge + BoardModel.BLOCK_SIZE * y;
	return pos;
end

function BoardModel.get_tile_board_position(x,y)
	local pos = vmath.vector3();
	pos.x = math.floor((x - side_edge + BoardModel.BLOCK_SIZE / 2) / BoardModel.BLOCK_SIZE);
	pos.y = math.floor((y - bottom_edge + BoardModel.BLOCK_SIZE / 2) / BoardModel.BLOCK_SIZE);
	if pos.x < 0 or pos.y < 0 then return nil; end
	return pos;
end

function BoardModel:new(id)	
	instance = {};
	setmetatable(instance, self);
	self.__index = self;

	local tiles = {};
	local _id = id;

	function instance.load_board(self, board_data)
		for _, new_tile in pairs(board_data) do
			local color = new_tile.color;
			local pos = new_tile.pos;
			if color then
				self:update_tile_color(pos.x, pos.y, vmath.vector4(color.r, color.g, color.b, color.a));
			end
		end
	end

	function instance.save_curent_board(self)
		local saveData = {};
		local tile = nil;
		for x = 0,  BoardModel.BOARD_TILES_HOR - 1 do
			for y = 0, BoardModel.BOARD_TILES_VERT - 1 do
				tile = tiles[x][y];
				local t = {
					pos = tile.pos,
					color = {
						r = tile.color.x,
						g = tile.color.y,
						b = tile.color.z,
						a = tile.color.w
					} 
				}
				table.insert(saveData, t);
			end
		end
		return saveData;
	end

	function instance.update_tile_color(self, x, y, color)
		local tile = self:get_tile(x, y);
		if tile and tile ~= "empty" then
			tile.color = color;
			msg.post(tile.controller_url, "set_color", {tile_color = tile.color});	
		end
	end
	
	function instance.create_tile(self, x, y, color)
		local scene_pos = BoardModel.get_tile_world_position(x, y);
		local factory_url = msg.url(nil, _id,  hash("TileFactory"));
		local tile_id = factory.create(factory_url, scene_pos, nil, {tile_color = color});
		local url = msg.url(nil, tile_id, "TileController");
		local tile_model = TileVO:new(tile_id, url, color, {x = x, y = y});
		msg.post(url, "set_board_url");
		self:add_tile(tile_model);
		go.set_parent(tile_id, _id)
		return tile_model;
	end

	function instance.add_tile(self, tile_model)
		if tile_model.pos.x < 0 or tile_model.pos.x >= BoardModel.BOARD_TILES_HOR then return nil end
		if tile_model.pos.y < 0 or tile_model.pos.y >= BoardModel.BOARD_TILES_VERT then return nil end

		if tiles[tile_model.pos.x] == nil then tiles[tile_model.pos.x] = {} end

		tiles[tile_model.pos.x][tile_model.pos.y] = tile_model;

		return tile_model;
	end

	function instance.remove_tile(self, x, y)
		if x < 0 or x >= BoardModel.BOARD_TILES_HOR then return nil end
		if y < 0 or y >= BoardModel.BOARD_TILES_VERT then return nil end
		local tile = tiles[x][y];
		tiles[x][y] = nil;
		return tile
	end

	function instance.replace_tile(self, x, y, tile_model)
		if x < 0 or x >= BoardModel.BOARD_TILES_HOR then return end
		if y < 0 or y >= BoardModel.BOARD_TILES_VERT then return end
		self:remove_tile(x, y);
		tile_model.pos.x, tile_model.pos.y = x, y;
		self:add_tile(tile_model);
	end

	function instance.get_tile(self, x, y)
		if tiles[x] then 
			return tiles[x][y];
		end
		return nil;
	end

	-- function instance.reset_tile(self, x, y);
	-- 	local tile = self:get_tile(x, y);
	-- 	if tile then
	-- 		msg.post(tile.url, "set_board_url");
	-- 	end
	-- end
	
	return instance;
end

return BoardModel;

