local Button = {}
local pressed = "_pressed";
local released = "_released";
local pressed_scale = vmath.vector3(0.85, 0.85, 0);
local released_scale = vmath.vector3(1, 1, 0);

function Button:new(node, pressed_animation, released_animation, callback)
	local instance = {
		is_over = false;
		is_pressed = false;
		gui_node = node;
		id = gui.get_id(node);
		current_anim = gui.get_flipbook(node);
		pressed_anim = pressed_animation;
		released_anim = released_animation;
		over_anim = nil;
	}
	
	setmetatable(instance, self);
	self.__index = self;

	local function setOverAnim(self, node)
		if self.over_anim and self.current_anim ~= self.over_anim then
			gui.play_flipbook(node, self.over_anim);
			self.current_anim = self.over_anim;
		end
	end

	local function setReleasedAnim(self, node)
		if self.released_anim and self.current_anim ~= self.released_anim then
			gui.play_flipbook(node, self.released_anim);
			self.current_anim = self.released_anim;
		end
		gui.set_scale(node, released_scale);
		self.pressed = false;
	end

	local function setPressedAnim(self, node)
		if self.pressed_anim and self.current_anim ~= self.pressed_anim then
			gui.play_flipbook(node, self.pressed_anim);
			self.current_anim = self.pressed_anim;
		end
		gui.set_scale(node, pressed_scale);
		self.pressed = true;
	end

	function instance:on_input(action_id, action)
		local over = gui.pick_node(node, action.x, action.y);

		if over and action.pressed and not self.pressed then
			setPressedAnim(self, self.gui_node);
		elseif action.released and self.pressed then
			setReleasedAnim(self, self.gui_node);
			if over then callback(self); end
		end

		if action_id == nil then
			-- if over then 
			-- 	if not self.is_over then
			-- 		setOverAnim(self, self.gui_node)
			-- 		self.is_over = true
			-- 	end
			-- else
			-- 	if self.is_over then
			-- 		-- gui.set_text(self.text, "BUTTON")
			-- 		self.is_over = false
			-- 	end
			-- end
		end
	end
	
	setReleasedAnim(instance, node);
	return instance;
end
return Button;


-- function init(self)
-- 	msg.post(".", "acquire_input_focus") -- <1>
-- 	self.button = gui.get_node("button") -- <2>
-- 	self.text = gui.get_node("text") -- <3>
-- 	self.is_over = false -- <4>
-- end
-- 
-- function on_input(self, action_id, action)
-- 	if action_id == nil then --<5>
-- 		if gui.pick_node(self.button, action.x, action.y) then -- <6>
-- 			if not self.is_over then
-- 				gui.set_text(self.text, "HELLO!") -- <7>
-- 				self.is_over = true
-- 			end
-- 		else
-- 			if self.is_over then
-- 				gui.set_text(self.text, "BUTTON") -- <8>
-- 				self.is_over = false
-- 			end
-- 		end
-- 	end
-- end

